@@ -3,21 +3,66 @@ from __future__ import unicode_literals
 
 from itertools import chain
 
+from django.apps import apps
+from django.conf import settings
 from django.contrib.admin.utils import (
     NotRelationField, flatten, get_fields_from_path,
 )
 from django.core import checks
 from django.core.exceptions import FieldDoesNotExist
 from django.db import models
+from django.db.models.constants import LOOKUP_SEP
 from django.forms.models import (
     BaseModelForm, BaseModelFormSet, _get_foreign_key,
 )
+from django.template.engine import Engine
 
 
-def check_admin_app(**kwargs):
-    from django.contrib.admin.sites import system_check_errors
+def check_admin_app(app_configs, **kwargs):
+    from django.contrib.admin.sites import all_sites
+    errors = []
+    for site in all_sites:
+        errors.extend(site.check(app_configs))
+    return errors
 
-    return system_check_errors
+
+def check_dependencies(**kwargs):
+    """
+    Check that the admin's dependencies are correctly installed.
+    """
+    errors = []
+    # contrib.contenttypes must be installed.
+    if not apps.is_installed('django.contrib.contenttypes'):
+        missing_app = checks.Error(
+            "'django.contrib.contenttypes' must be in INSTALLED_APPS in order "
+            "to use the admin application.",
+            id="admin.E401",
+        )
+        errors.append(missing_app)
+    # The auth context processor must be installed if using the default
+    # authentication backend.
+    try:
+        default_template_engine = Engine.get_default()
+    except Exception:
+        # Skip this non-critical check:
+        # 1. if the user has a non-trivial TEMPLATES setting and Django
+        #    can't find a default template engine
+        # 2. if anything goes wrong while loading template engines, in
+        #    order to avoid raising an exception from a confusing location
+        # Catching ImproperlyConfigured suffices for 1. but 2. requires
+        # catching all exceptions.
+        pass
+    else:
+        if ('django.contrib.auth.context_processors.auth'
+                not in default_template_engine.context_processors and
+                'django.contrib.auth.backends.ModelBackend' in settings.AUTHENTICATION_BACKENDS):
+            missing_template = checks.Error(
+                "'django.contrib.auth.context_processors.auth' must be in "
+                "TEMPLATES in order to use the admin application.",
+                id="admin.E402"
+            )
+            errors.append(missing_template)
+    return errors
 
 
 class BaseModelAdminChecks(object):
@@ -61,8 +106,8 @@ class BaseModelAdminChecks(object):
             return refer_to_missing_field(field=field_name, option=label,
                                           model=model, obj=obj, id='admin.E002')
         else:
-            if not isinstance(field, (models.ForeignKey, models.ManyToManyField)):
-                return must_be('a ForeignKey or ManyToManyField',
+            if not field.many_to_many and not isinstance(field, models.ForeignKey):
+                return must_be('a foreign key or a many-to-many field',
                                option=label, obj=obj, id='admin.E003')
             else:
                 return []
@@ -80,7 +125,6 @@ class BaseModelAdminChecks(object):
             return [
                 checks.Error(
                     "Both 'fieldsets' and 'fields' are specified.",
-                    hint=None,
                     obj=obj.__class__,
                     id='admin.E005',
                 )
@@ -90,7 +134,6 @@ class BaseModelAdminChecks(object):
             return [
                 checks.Error(
                     "The value of 'fields' contains duplicate field(s).",
-                    hint=None,
                     obj=obj.__class__,
                     id='admin.E006',
                 )
@@ -129,7 +172,6 @@ class BaseModelAdminChecks(object):
             return [
                 checks.Error(
                     "The value of '%s[1]' must contain the key 'fields'." % label,
-                    hint=None,
                     obj=obj.__class__,
                     id='admin.E011',
                 )
@@ -142,7 +184,6 @@ class BaseModelAdminChecks(object):
             return [
                 checks.Error(
                     "There are duplicate field(s) in '%s[1]'." % label,
-                    hint=None,
                     obj=obj.__class__,
                     id='admin.E012',
                 )
@@ -183,10 +224,9 @@ class BaseModelAdminChecks(object):
                         not field.remote_field.through._meta.auto_created):
                     return [
                         checks.Error(
-                            ("The value of '%s' cannot include the ManyToManyField '%s', "
-                             "because that field manually specifies a relationship model.")
+                            "The value of '%s' cannot include the ManyToManyField '%s', "
+                            "because that field manually specifies a relationship model."
                             % (label, field_name),
-                            hint=None,
                             obj=obj.__class__,
                             id='admin.E013',
                         )
@@ -205,7 +245,6 @@ class BaseModelAdminChecks(object):
             return [
                 checks.Error(
                     "The value of 'exclude' contains duplicate field(s).",
-                    hint=None,
                     obj=obj.__class__,
                     id='admin.E015',
                 )
@@ -258,8 +297,8 @@ class BaseModelAdminChecks(object):
             return refer_to_missing_field(field=field_name, option=label,
                                           model=model, obj=obj, id='admin.E019')
         else:
-            if not isinstance(field, models.ManyToManyField):
-                return must_be('a ManyToManyField', option=label, obj=obj, id='admin.E020')
+            if not field.many_to_many:
+                return must_be('a many-to-many field', option=label, obj=obj, id='admin.E020')
             else:
                 return []
 
@@ -294,7 +333,6 @@ class BaseModelAdminChecks(object):
                         "instance of ForeignKey, and does not have a 'choices' definition." % (
                             label, field_name
                         ),
-                        hint=None,
                         obj=obj.__class__,
                         id='admin.E023',
                     )
@@ -311,7 +349,6 @@ class BaseModelAdminChecks(object):
             return [
                 checks.Error(
                     "The value of '%s' must be either admin.HORIZONTAL or admin.VERTICAL." % label,
-                    hint=None,
                     obj=obj.__class__,
                     id='admin.E024',
                 )
@@ -325,7 +362,6 @@ class BaseModelAdminChecks(object):
                 return [
                     checks.Error(
                         "The value of 'view_on_site' must be a callable or a boolean value.",
-                        hint=None,
                         obj=obj.__class__,
                         id='admin.E025',
                     )
@@ -355,26 +391,17 @@ class BaseModelAdminChecks(object):
         is a name of existing field and the field is one of the allowed types.
         """
 
-        forbidden_field_types = (
-            models.DateTimeField,
-            models.ForeignKey,
-            models.ManyToManyField
-        )
-
         try:
             field = model._meta.get_field(field_name)
         except FieldDoesNotExist:
             return refer_to_missing_field(field=field_name, option=label,
                                           model=model, obj=obj, id='admin.E027')
         else:
-            if isinstance(field, forbidden_field_types):
+            if isinstance(field, (models.DateTimeField, models.ForeignKey, models.ManyToManyField)):
                 return [
                     checks.Error(
                         "The value of '%s' refers to '%s', which must not be a DateTimeField, "
-                        "ForeignKey or ManyToManyField." % (
-                            label, field_name
-                        ),
-                        hint=None,
+                        "a ForeignKey, a OneToOneField, or a ManyToManyField." % (label, field_name),
                         obj=obj.__class__,
                         id='admin.E028',
                     )
@@ -401,8 +428,7 @@ class BaseModelAdminChecks(object):
         try:
             model._meta.get_field(field_name)
         except FieldDoesNotExist:
-            return refer_to_missing_field(field=field_name, option=label,
-                                          model=model, obj=obj, id='admin.E030')
+            return refer_to_missing_field(field=field_name, option=label, model=model, obj=obj, id='admin.E030')
         else:
             return []
 
@@ -426,8 +452,8 @@ class BaseModelAdminChecks(object):
         if field_name == '?' and len(obj.ordering) != 1:
             return [
                 checks.Error(
-                    ("The value of 'ordering' has the random ordering marker '?', "
-                     "but contains other fields as well."),
+                    "The value of 'ordering' has the random ordering marker '?', "
+                    "but contains other fields as well.",
                     hint='Either remove the "?", or remove the other fields.',
                     obj=obj.__class__,
                     id='admin.E032',
@@ -435,19 +461,19 @@ class BaseModelAdminChecks(object):
             ]
         elif field_name == '?':
             return []
-        elif '__' in field_name:
+        elif LOOKUP_SEP in field_name:
             # Skip ordering in the format field1__field2 (FIXME: checking
             # this format would be nice, but it's a little fiddly).
             return []
         else:
             if field_name.startswith('-'):
                 field_name = field_name[1:]
-
+            if field_name == 'pk':
+                return []
             try:
                 model._meta.get_field(field_name)
             except FieldDoesNotExist:
-                return refer_to_missing_field(field=field_name, option=label,
-                                              model=model, obj=obj, id='admin.E033')
+                return refer_to_missing_field(field=field_name, option=label, model=model, obj=obj, id='admin.E033')
             else:
                 return []
 
@@ -480,7 +506,6 @@ class BaseModelAdminChecks(object):
                         "The value of '%s' is not a callable, an attribute of '%s', or an attribute of '%s.%s'." % (
                             label, obj.__class__.__name__, model._meta.app_label, model._meta.object_name
                         ),
-                        hint=None,
                         obj=obj.__class__,
                         id='admin.E035',
                     )
@@ -540,13 +565,12 @@ class ModelAdminChecks(BaseModelAdminChecks):
         """ Check one inline model admin. """
         inline_label = '.'.join([inline.__module__, inline.__name__])
 
-        from django.contrib.admin.options import BaseModelAdmin
+        from django.contrib.admin.options import InlineModelAdmin
 
-        if not issubclass(inline, BaseModelAdmin):
+        if not issubclass(inline, InlineModelAdmin):
             return [
                 checks.Error(
-                    "'%s' must inherit from 'BaseModelAdmin'." % inline_label,
-                    hint=None,
+                    "'%s' must inherit from 'InlineModelAdmin'." % inline_label,
                     obj=obj.__class__,
                     id='admin.E104',
                 )
@@ -555,14 +579,12 @@ class ModelAdminChecks(BaseModelAdminChecks):
             return [
                 checks.Error(
                     "'%s' must have a 'model' attribute." % inline_label,
-                    hint=None,
                     obj=obj.__class__,
                     id='admin.E105',
                 )
             ]
         elif not issubclass(inline.model, models.Model):
-            return must_be('a Model', option='%s.model' % inline_label,
-                           obj=obj, id='admin.E106')
+            return must_be('a Model', option='%s.model' % inline_label, obj=obj, id='admin.E106')
         else:
             return inline(model, obj.admin_site).check()
 
@@ -600,7 +622,6 @@ class ModelAdminChecks(BaseModelAdminChecks):
                         "callable, an attribute of '%s', or an attribute or method on '%s.%s'." % (
                             label, item, obj.__class__.__name__, model._meta.app_label, model._meta.object_name
                         ),
-                        hint=None,
                         obj=obj.__class__,
                         id='admin.E108',
                     )
@@ -609,7 +630,6 @@ class ModelAdminChecks(BaseModelAdminChecks):
                 return [
                     checks.Error(
                         "The value of '%s' must not be a ManyToManyField." % label,
-                        hint=None,
                         obj=obj.__class__,
                         id='admin.E109',
                     )
@@ -628,7 +648,6 @@ class ModelAdminChecks(BaseModelAdminChecks):
                         "an attribute of '%s', or an attribute or method on '%s.%s'." % (
                             label, item, obj.__class__.__name__, model._meta.app_label, model._meta.object_name
                         ),
-                        hint=None,
                         obj=obj.__class__,
                         id='admin.E108',
                     )
@@ -639,16 +658,21 @@ class ModelAdminChecks(BaseModelAdminChecks):
     def _check_list_display_links(self, obj):
         """ Check that list_display_links is a unique subset of list_display.
         """
+        from django.contrib.admin.options import ModelAdmin
 
         if obj.list_display_links is None:
             return []
         elif not isinstance(obj.list_display_links, (list, tuple)):
             return must_be('a list, a tuple, or None', option='list_display_links', obj=obj, id='admin.E110')
-        else:
+        # Check only if ModelAdmin.get_list_display() isn't overridden.
+        elif obj.get_list_display.__code__ is ModelAdmin.get_list_display.__code__:
+            # Use obj.get_list_display.__func__ is ModelAdmin.get_list_display
+            # when dropping PY2.
             return list(chain(*[
                 self._check_list_display_links_item(obj, field_name, "list_display_links[%d]" % index)
                 for index, field_name in enumerate(obj.list_display_links)
             ]))
+        return []
 
     def _check_list_display_links_item(self, obj, field_name, label):
         if field_name not in obj.list_display:
@@ -657,7 +681,6 @@ class ModelAdminChecks(BaseModelAdminChecks):
                     "The value of '%s' refers to '%s', which is not defined in 'list_display'." % (
                         label, field_name
                     ),
-                    hint=None,
                     obj=obj.__class__,
                     id='admin.E111',
                 )
@@ -695,7 +718,6 @@ class ModelAdminChecks(BaseModelAdminChecks):
                 return [
                     checks.Error(
                         "The value of '%s' must not inherit from 'FieldListFilter'." % label,
-                        hint=None,
                         obj=obj.__class__,
                         id='admin.E114',
                     )
@@ -706,8 +728,7 @@ class ModelAdminChecks(BaseModelAdminChecks):
             # item is option #2
             field, list_filter_class = item
             if not issubclass(list_filter_class, FieldListFilter):
-                return must_inherit_from(parent='FieldListFilter', option='%s[1]' % label,
-                                         obj=obj, id='admin.E115')
+                return must_inherit_from(parent='FieldListFilter', option='%s[1]' % label, obj=obj, id='admin.E115')
             else:
                 return []
         else:
@@ -721,7 +742,6 @@ class ModelAdminChecks(BaseModelAdminChecks):
                 return [
                     checks.Error(
                         "The value of '%s' refers to '%s', which does not refer to a Field." % (label, field),
-                        hint=None,
                         obj=obj.__class__,
                         id='admin.E116',
                     )
@@ -733,8 +753,7 @@ class ModelAdminChecks(BaseModelAdminChecks):
         """ Check that list_select_related is a boolean, a list or a tuple. """
 
         if not isinstance(obj.list_select_related, (bool, list, tuple)):
-            return must_be('a boolean, tuple or list', option='list_select_related',
-                           obj=obj, id='admin.E117')
+            return must_be('a boolean, tuple or list', option='list_select_related', obj=obj, id='admin.E117')
         else:
             return []
 
@@ -770,15 +789,13 @@ class ModelAdminChecks(BaseModelAdminChecks):
         try:
             field = model._meta.get_field(field_name)
         except FieldDoesNotExist:
-            return refer_to_missing_field(field=field_name, option=label,
-                                          model=model, obj=obj, id='admin.E121')
+            return refer_to_missing_field(field=field_name, option=label, model=model, obj=obj, id='admin.E121')
         else:
             if field_name not in obj.list_display:
                 return [
                     checks.Error(
                         "The value of '%s' refers to '%s', which is not "
                         "contained in 'list_display'." % (label, field_name),
-                        hint=None,
                         obj=obj.__class__,
                         id='admin.E122',
                     )
@@ -787,22 +804,20 @@ class ModelAdminChecks(BaseModelAdminChecks):
                 return [
                     checks.Error(
                         "The value of '%s' cannot be in both 'list_editable' and 'list_display_links'." % field_name,
-                        hint=None,
                         obj=obj.__class__,
                         id='admin.E123',
                     )
                 ]
-            # Check that list_display_links is set, and that the first values of list_editable and list_display are
-            # not the same. See ticket #22792 for the use case relating to this.
-            elif (obj.list_display[0] in obj.list_editable and obj.list_display[0] != obj.list_editable[0] and
-                  obj.list_display_links is not None):
+            # If list_display[0] is in list_editable, check that
+            # list_display_links is set. See #22792 and #26229 for use cases.
+            elif (obj.list_display[0] == field_name and not obj.list_display_links and
+                    obj.list_display_links is not None):
                 return [
                     checks.Error(
                         "The value of '%s' refers to the first field in 'list_display' ('%s'), "
                         "which cannot be used unless 'list_display_links' is set." % (
                             label, obj.list_display[0]
                         ),
-                        hint=None,
                         obj=obj.__class__,
                         id='admin.E124',
                     )
@@ -813,7 +828,6 @@ class ModelAdminChecks(BaseModelAdminChecks):
                         "The value of '%s' refers to '%s', which is not editable through the admin." % (
                             label, field_name
                         ),
-                        hint=None,
                         obj=obj.__class__,
                         id='admin.E125',
                     )
@@ -836,15 +850,19 @@ class ModelAdminChecks(BaseModelAdminChecks):
             return []
         else:
             try:
-                field = obj.model._meta.get_field(obj.date_hierarchy)
-            except FieldDoesNotExist:
-                return refer_to_missing_field(option='date_hierarchy',
-                                              field=obj.date_hierarchy,
-                                              model=obj.model, obj=obj, id='admin.E127')
+                field = get_fields_from_path(obj.model, obj.date_hierarchy)[-1]
+            except (NotRelationField, FieldDoesNotExist):
+                return [
+                    checks.Error(
+                        "The value of 'date_hierarchy' refers to '%s', which "
+                        "does not refer to a Field." % obj.date_hierarchy,
+                        obj=obj.__class__,
+                        id='admin.E127',
+                    )
+                ]
             else:
                 if not isinstance(field, (models.DateField, models.DateTimeField)):
-                    return must_be('a DateField or DateTimeField', option='date_hierarchy',
-                                   obj=obj, id='admin.E128')
+                    return must_be('a DateField or DateTimeField', option='date_hierarchy', obj=obj, id='admin.E128')
                 else:
                     return []
 
@@ -884,7 +902,6 @@ class InlineModelAdminChecks(BaseModelAdminChecks):
                     "to the parent model '%s.%s'." % (
                         fk.name, parent_model._meta.app_label, parent_model._meta.object_name
                     ),
-                    hint=None,
                     obj=obj.__class__,
                     id='admin.E201',
                 )
@@ -896,7 +913,7 @@ class InlineModelAdminChecks(BaseModelAdminChecks):
         try:
             _get_foreign_key(parent_model, obj.model, fk_name=obj.fk_name)
         except ValueError as e:
-            return [checks.Error(e.args[0], hint=None, obj=obj.__class__, id='admin.E202')]
+            return [checks.Error(e.args[0], obj=obj.__class__, id='admin.E202')]
         else:
             return []
 
@@ -932,8 +949,7 @@ class InlineModelAdminChecks(BaseModelAdminChecks):
         """ Check formset is a subclass of BaseModelFormSet. """
 
         if not issubclass(obj.formset, BaseModelFormSet):
-            return must_inherit_from(parent='BaseModelFormSet', option='formset',
-                                     obj=obj, id='admin.E206')
+            return must_inherit_from(parent='BaseModelFormSet', option='formset', obj=obj, id='admin.E206')
         else:
             return []
 
@@ -942,7 +958,6 @@ def must_be(type, option, obj, id):
     return [
         checks.Error(
             "The value of '%s' must be %s." % (option, type),
-            hint=None,
             obj=obj.__class__,
             id=id,
         ),
@@ -953,7 +968,6 @@ def must_inherit_from(parent, option, obj, id):
     return [
         checks.Error(
             "The value of '%s' must inherit from '%s'." % (option, parent),
-            hint=None,
             obj=obj.__class__,
             id=id,
         ),
@@ -966,7 +980,6 @@ def refer_to_missing_field(field, option, model, obj, id):
             "The value of '%s' refers to '%s', which is not an attribute of '%s.%s'." % (
                 option, field, model._meta.app_label, model._meta.object_name
             ),
-            hint=None,
             obj=obj.__class__,
             id=id,
         ),
