@@ -1,12 +1,9 @@
 from __future__ import unicode_literals
 
-from django.contrib.gis.gdal import HAS_GDAL
+from django.contrib.gis.gdal import CoordTransform, SpatialReference
 from django.core.serializers.base import SerializerDoesNotExist
 from django.core.serializers.json import Serializer as JSONSerializer
 
-if HAS_GDAL:
-    from django.contrib.gis.gdal import CoordTransform, SpatialReference
-
 
 class Serializer(JSONSerializer):
     """
